from .classes import Input
from .functions import create_data, create_output

class KNNClassifier(object):

	def __init__(self, xs, ys, labels):

		self.train_x = xs
		self.train_y = ys
		self.labels  = labels

		# train data as Data object
		self.data = create_data(self.train_x, self.train_y, self.labels)

	def predict(self, input_):

		if type(input_) != tuple or len(input_) != 2:
			raise Exception("input format is not appropriate")

		# input as an Input object
		self.input = Input(input_[0], input_[1])

		# output data according to input and train data
		self.output = create_output(self.input, self.data)

		# calculated euclidean distance
		self.distance = [self.output[i].dist for i in range(len(self.output))]

		# prediction according to euclidean distance calculation
		min_distance = self.distance.index(min(self.distance))
		prediction   = self.output[min_distance].label

		return prediction


import os
import pandas as pd

# show data from the csv file
def show_data(dataframe, row_number : int = 10):
	dataframe.head(row_number)

# drop NaN cells
def drop_nan(dataframe, axis : int, threshold : int):
	try:
	        dropped_dataframe = dataframe.dropna(axis = axis, thresh = threshold)
	except Exception as e:
		print(e)

	return dropped_dataframe

# load files from a file
def load_data(path : str, filename : str):
	if ".csv" not in filename:
		raise Exception("File format must be a '.csv'")

	try:
		filepath = os.path.join(path, filename)
		file = pd.read_csv(filepath)
		data = pd.DataFrame(file)
	except Exception as e:
		print(e)

	return data



import numpy as np
from .classes import Input, Data, Output

# euclidean distance
def euc_dist(x, y, a, b):
	return np.sqrt((x - a) ** 2 + (y - b) ** 2)

# create data - returns list of Data Objects
def create_data(xs : list, ys : list, labels : list):
	try:
		all_data = list()
		for i in range(len(xs)):
			data = Data(xs[i], ys[i], labels[i])
			all_data.append(data)
	except Exception as e:
		print(e)

	return all_data

# create output list - returns list of Output object
def create_output(input_, data):
	try:
		output = list()
		for i in range(len(data)):
			distance = euc_dist(input_.x, input_.y, data[i].x, data[i].y)
			label = data[i].label
			output_data = Output(distance, label)
			output.append(output_data)
	except Exception as e:
		print(e)

	return output


import numpy as np

class Input(object):

	def __init__(self, x, y):
		self.x = np.array([x]).astype(np.float32)
		self.y = np.array([y]).astype(np.float32)

	def __repr__(self):
		return f"<class: Input, (x = {self.x}, y = {self.y})>"

class Data(object):

	def __init__(self, x, y, label):
		self.x = np.array([x]).astype(np.float32)
		self.y = np.array([y]).astype(np.float32)
		self.label = label

	def __repr__(self):
		return f"<class: Data>"

class Output(object):

	def __init__(self, distance, label):
		self.dist = np.array([distance]).astype(np.float32)
		self.label = label

	def __repr__(self):
		return f"<class: Output>"


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="basic_knn",
    version="0.0.2",
    author="Ege Sabancı",
    author_email="egesabanci@outlook.com.tr",
    description="K-Nearest Neighbors algorithm for classification problems.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Egesabanci/basic_knn",
    download_url = "https://pypi.org/project/basic_knn",
    packages=setuptools.find_packages(),
    install_requires=["numpy", "pandas"],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)